==> rflowcyt_1.4.0.tar.gz: rflowcyt/R/ImageParCoord.R <==
ImageParCoord <- function(x, ## x is a matrix
                           ##   num.cells=1:10,  this index is not needed
                           num.bins=10,
                           range.var=range(x),
                           break10 = NULL,
                           joint=FALSE,
                           title="",
                           use.shortnames=FALSE,
                           color.image=gray((25:5/25)[-c(1,2,3, 4, 5, 6)]),
                                        #rainbow(16, start=.4, end=.1),#
                           xwidth.scale=5,
                           ntrans=1,
                           legend.plotted=TRUE,
                           hist.plotted=FALSE,
                           image.plotted=TRUE,
                           para.plotted=FALSE,
                           lines.plotted=TRUE,
                           lwd.vec=1:7,
                           lty.vec=rep(1,7),
                           col.vec=7:1,
                           range.image=c(0,dim(x)[1]),
                           shrink.legend=TRUE,
                           horizontal.legend = TRUE,
                           offset.legend=0.03,
                           nlevel.legend=length(color.image),
                           xlab.image="",
                           ylab.image="Bins",
                           MY.DEBUG=TRUE,...){
    range.var <-range(x)
    break10 <- defineBreak(x,num.bins,break10=break10,MY.DEBUG=TRUE)
    max.lwd <- max(lwd.vec)
    variable.names <- colnames(x)
    nvar <- dim(x)[2]

    if (nvar < 2){
        stop("nvar should be equal to or greater than 2.")
    }
    
    
    imageBlockinfo <- imageBlock(x=x,num.bins=num.bins,ntrans=ntrans,xwidth.scale=xwidth.scale,hist.plotted=hist.plotted,breaks=break10)
    image.block <- imageBlockinfo[[1]]
    coords <- imageBlockinfo[[2]]
    V1.hist <- imageBlockinfo[[3]]

    varlabpos <- vector()
    for (i in 1:nvar){
        curpos <- ifelse(i==1, (ntrans*xwidth.scale)/2, (ntrans*xwidth.scale)/2 + 2*(i-1)*ntrans*xwidth.scale )
        varlabpos <- c(varlabpos, curpos)
    }

    if (image.plotted==TRUE){
        ## display the image plot
        display.imagepl(image.block,V1.hist,
                        varlabpos,
                        variable.names,
                        zlim=range.image,
                        col=color.image,
                        legend.shrink,
                        horizontal = horizontal.legend,
                        offset=offset.legend,
                        shrink=shrink.legend,
                        nlevel=nlevel.legend,
                        xlab=xlab.image,
                        ylab=ylab.image,
                        main=title)
    } 


    if (joint==FALSE){
        info <- list()
        lines.ls <- list()
        for (varcol in 2:nvar){
            
            first <- varcol-1
            second <- varcol
            
            V1.coord <- coords[,first]
            V2.coord <- coords[,second]
            tab<-table(V1.coord, V2.coord)
            
            
            rows <- sort(unique(V1.coord))
            cols <- sort(unique(V2.coord))
            
            twocoord <- NULL
            for (c in 1:length(cols)){
                for (r in 1: length(rows)){
                    twocoord <- rbind(twocoord, c(rows[r], cols[c]))
                }
            }
            
            colnames(twocoord) <- c(paste("V", first, ".bin", sep=""), paste("V", second, ".bin", sep=""))
            
            pos <- which(tab!=0)
            num.lines <- tab[pos]       
            coords.nozero <- twocoord[pos,]
            
            info[[varcol-1]] <- cbind(coords.nozero, num.lines)
            rownames(info[[varcol-1]]) <- NULL
            lines.ls[[varcol-1]]<- num.lines
        }
    }

    if (joint==TRUE){
        
        ## sort the last element of the list
        sort.pos <- sort(coords[[nvar]], decreasing=FALSE, index.return=TRUE)$ix
        coords <- sortL(coords, sort.pos)
        tab <- table(coords)
        coords.mat <- matrix(unlist(coords), ncol=nvar)
        var.pat <- unique(coords.mat)
        
        dim.name <- labels(tab)
        num.types <- lapply(dim.name, function(w){length(w)})
        ## get the position of the row in the
        ## corresponding element in the list dim.name
                
        dimensions <- t(apply(var.pat, 1, function(row){
            pos <- getPos(row, dim.name)
            
        }))
        
        ## convert the dimensions into the real positions in a vector
        ## please note that R converts lists and matrices into vectors
        ## by column, or else this following procedure will be wrong!
        real.pos <- NULL
        for (row in 1:(dim(dimensions)[1])){
            value <- dimensions[row,1]
            mult <- 1
            for (i in 1:(length(num.types)-1)){
                mult <- mult*num.types[[i]]
                value <- value + (dimensions[row,i+1]-1)*mult
            }
            real.pos <- c(real.pos, value)
        }
        
        ## transform the table into a vector: R does this
        ## by column (not by row...) using as.vector
        
        tab.vec <- as.vector(tab)
        num.lines <- tab.vec[real.pos]
        
        line.info<- cbind(var.pat, num.lines)
        
        
        info <- list()
        lines.ls <- list()
        first.var <- line.info[,1]
        
        for (pair in 2:(dim(var.pat)[2])){
            second.var <- line.info[,pair]
            info[[pair-1]] <- cbind(first.var, second.var, num.lines)
            lines.ls[[pair-1]] <- num.lines
            first.var <- second.var
        }
    }
    ## get the max number of lines
    max.size <- max(num.lines)
    min.size <- min(num.lines)

    
    plotLine(max.size,min.size,info=info,varlabpos,lwd.vec,lty.vec,col.vec,lines.plotted=lines.plotted,lines.ls,image.plotted=image.plotted)


    if (legend.plotted) {
        plot(1:range.var[2],1:range.var[2], xlab="", ylab="", type="n", main="Key for counts signified by the line", xaxt="n", yaxt="n" )
        legend(range.var[1], range.var[2], paste(cts.ls, "<= Counts <=", cts.ls2), lwd=lwd.vec, col=col.vec, lty=lty.vec, bg=color.image[1])
    }    
    ## real parallel coordinates
    if(para.plotted==TRUE){
        parallelCoordinates(x,
                             varlabpos=varlabpos, variable.names=variable.names,
                             at.y=round(V1.hist$breaks, 1)[-1], superimpose=FALSE)
        
        box()
        title(main = title, font.main = 4)
        abline(v=varlabpos)
    }
    
    if (is.null(info)==TRUE){
        return(list(image.block=image.block, breaks=break10))
    } else{
        return(list(image.block=image.block, line.info=info, breaks=break10))
    }
}
 

############################################################################################################################################

defineBreak <- function(x,num.bins,break10=NULL,MY.DEBUG=TRUE){

    if (is.null(break10)){
        break10 <- seq(range.var[1]-1, range.var[2], by=(range.var[2]/num.bins)) 
    }
   ## num.bins is the number of bins for the image plot
   if(num.bins!=(length(break10)-1)){
       if (MY.DEBUG){
           cat("\n\n", "Number of bins, num.bins, has been changed from", num.bins, "to",
               length(break10)-1, "\n\n")
       }
       num.bins <- length(break10)-1
   }
   
   if( break10[length(break10)]<range.var[2] ){
       if (MY.DEBUG){
           cat("\n\n", "The following breaks for the binning:","\n")
           print(break10)
           cat("should be inclusive of the range:", "\n")
           print(range.var)
       }
       seq1 <- seq(range.var[1]-1, range.var[2], by=range.var[2]/num.bins)
       break10 <- c(seq1[-length(seq1)], range.var[2])
       if (MY.DEBUG){
           cat("\n\n", "Thus, the breaks, break10, has been changed to:", "\n")
           print(break10)
       }
   }
    return(break10)
   
}

imageBlock <- function(x,num.bins,ntrans,xwidth.scale,hist.plotted=hist.plotted,breaks){
     image.block <- coords<- NULL
     nvar <- dim(x)[2]
     variable.names <- colnames(x)
     ## check that ntrans is bigger than 1
     if (ntrans <1) {
         stop("ntrans should be greater than or equal to 1")
     }
  
     trans.ct <- rep(0, num.bins*xwidth.scale) ## initialize the transition count to zeros

     V1 <- x[, 1]
     V1.hist <- hist(V1, main=paste("Histogram of", variable.names[1], sep=" "),
                    xlab=variable.names[1], breaks=breaks, plot=hist.plotted, freq=TRUE)
     V1.ct <- V1.hist$counts
     V1.block<-matrix(rep(V1.ct, ntrans*xwidth.scale), ncol=ntrans*xwidth.scale)
     xcoord <- 1:(2+ntrans)
     V1.xpos <- 1
     V1.breaks <- V1.hist$breaks[-1]

     for (second in 2:nvar){
         
       V2 <- x[, second]
       ## histograms with 10 bins
       V2.hist <- hist(V2, main=paste("Histogram of", variable.names[second], sep=" "),
                       xlab=variable.names[second], breaks=breaks, plot=FALSE)
       V2.ct <- V2.hist$counts
    
       ## make the variable image blocks
       V2.block<-matrix(rep(V2.ct, ntrans*xwidth.scale), ncol=ntrans*xwidth.scale)

       ## for each cell/row observation make a slope line formula and
       ## predict the vertical bins (y-coordinate) that the observations
       ## lie depending on the position of the horizontal bin (x-coordinate)
       
       ## make x-coordinates
       ## only does 2 variables right now
       V2.xpos <- length(xcoord)
       xdiff <- V2.xpos-V1.xpos
  
       ## predict the y-coordinates from the x-coordinates
       ycoord.trans <- apply(cbind(V1, V2), 1, function(row){
           ydiff <- row[2]-row[1]
           slope <- ydiff/xdiff
           ypred <- sapply(xcoord[-c(V1.xpos, V2.xpos)], function(x){
               return(slope*(x-V1.xpos)+row[1])
           })
           return(round(ypred,0))
       })
    
       ycoord.trans <- matrix(unlist(ycoord.trans), ncol=length(xcoord[-c(V1.xpos, V2.xpos)]), byrow=TRUE)
       ## get the trans hist for each trans
       trans.block <- NULL
       for (i in 1:ntrans){
           trans.hist <- hist(ycoord.trans[,i], main=paste("Histogram of trans", i, sep=""),
                              xlab=variable.names[2], breaks=breaks, plot=hist.plotted, freq=TRUE)
           trans.block <- cbind(trans.block, matrix(rep(trans.hist$counts, xwidth.scale), ncol=xwidth.scale))
       }
       
       if (hist.plotted==TRUE){
           V2.hist <- hist(V2, main=paste("Histogram of", variable.names[second], sep=" "),
                           xlab=variable.names[second], breaks=break10, plot=TRUE, freq=TRUE)
           
       }
       ## make the total image block
       image.block <- cbind(image.block, V1.block, trans.block)

       V2.breaks <- V2.hist$breaks[-1]
       ## to center the coordinates in the blocks
       V1.coord <- getcoord(V1, V1.breaks) +0.5
       V2.coord <- getcoord(V2, V2.breaks) + 0.5
       coords <- cbind(coords, V1.coord)
       V1 <- V2
       
   }
     image.block <- cbind(image.block, V2.block)
     coords <- cbind(coords, V2.coord)
     
     return(list(image.block, coords,V1.hist))
 }


getcoord <- function(V, V.breaks){
    V.coord <- NULL
    for (i in 1:(length(V))) {
        ## j is the bin number for each element of V
        j <- 1
        while ((j < length(V.breaks)) & (V[i] >= V.breaks[j])){
            j <- j+1
        }
        
        V.coord <-c(V.coord, j)
        
    }
    return(V.coord)
}


 
display.imagepl <- function(image.block,V1.hist,
                            varlabpos,
                            variable.names,
                            zlim=range.image,
                            col=color.image,
                            legend.shrink,
                            horizontal = horizontal.legend,
                            offset=offset.legend,
                            nlevel=nlevel.legend,
                            shrink=shrink.legend,
                            xlab,
                            ylab,
                            main,...){
     ## coordinates of the image block
    col.coord<-1:(dim(image.block)[2]+1)
    row.coord <- 1:(dim(image.block)[1]+1)

    image(col.coord, row.coord, t(image.block), zlim=zlim, col=col, axes=FALSE,
          xlab=xlab, ylab=ylab, ...)
    lib.there <- require(fields)
    if (lib.there==FALSE){
        warning("The function image.plot in Library fields is needed for the image parallel coordinates plot to have a color legend.")
    } else {
        image.plot(col.coord, row.coord, t(image.block), zlim =zlim, legend.only = TRUE, legend.shrink=shrink,
                   horizontal = horizontal, col=col, offset=offset, nlevel=nlevel, axes=FALSE,
                   xlab=xlab, ylab=ylab, ...)
    }
    
    axis(2, at=1:length(V1.hist$breaks), lab=round(V1.hist$breaks, 1))
    axis(1, at=varlabpos, lab=variable.names)
    box()
    title(main, font.main = 4)
    abline(v=varlabpos)
}


plotLine <- function(max.size,min.size,varlabpos,info,lwd.vec,lty.vec,col.vec,lines.plotted=lines.plotted,lines.ls,image.plotted=lines.plotted){

    scaled.size.ls <- NULL
    max.lwd <- max(lwd.vec)
    
    ## plotting of the lines from the last variable to the first
    for(i in length(info):1){
        ## ordering to plot the large num lines first
        kk <- info[[i]]
        rownames(kk) <- NULL
        order.kk <- sort(kk[,3], decreasing=FALSE, index.return=TRUE)$ix
        kk <- kk[order.kk,]
        
        ## actual plotting of the lines
        scaled.size.el <- apply(kk, 1, function(y){
            scaled.size <- (max.lwd-1)*((y[3]-min.size)/(max.size-min.size))+1
            lwd.current <- round(scaled.size,0)
            pos <- which(lwd.vec==lwd.current)
            
            if (lines.plotted==TRUE) {
                if (image.plotted ==TRUE) {
                    lines(varlabpos[c(i,i+1)], c(y[1], y[2]), lwd=lwd.current,
                          col=col.vec[pos], lty=lty.vec[pos])
                } else {
                    warning("'lines.plotted'=TRUE, but lines could not be plotted because 'image.plotted'=FALSE")
                }
            }
            return(round(scaled.size, 0))
        })
        ##go back to the original order
        order.orig <- sort(order.kk, decreasing=FALSE, index.return=TRUE)$ix
        scaled.size.ls <- c(scaled.size.el[order.orig], scaled.size.ls)
    }
    lines.ls <- unlist(lines.ls)
    names(lines.ls) <- NULL
    
    cts.ls <- cts.ls2 <- NULL
    for (j in 1:max.lwd){
        if (is.element(j, scaled.size.ls)==TRUE){
            cts.max <-max(lines.ls[scaled.size.ls==j])
            cts.min <- min(lines.ls[scaled.size.ls==j])
            cts.ls <- c(cts.ls, cts.min)
            cts.ls2 <- c(cts.ls2, cts.max)
        } else {
            cts.ls <- c(cts.ls, "--")
            cts.ls2 <- c(cts.ls2, "--")
        }
    }
    
}

    
########## Joint 
sortL <- function(coords, sort.pos){
    lapply(coords, function(w){w[sort.pos]})
}

getPos <- function(row, dim.name){
    info <- cbind(row, 1:length(row))
    pos <- apply(info, 1, function(w){
        which(w[1]==as.numeric(dim.name[[w[2]]]))
    })
    return(unlist(pos))
    
}

==> rflowcyt_1.6.0.tar.gz: rflowcyt/R/ImageParCoord.R <==
ImageParCoord <- function(x, ## x is a matrix
                           ##num.cells=1:10,  this index is not needed
                           num.bins=10,
                           range.var=range(x),
                           break10 = NULL,
                           joint=FALSE,
                           title="",
                           use.shortnames=FALSE,
                           color.image=gray((25:5/25)[-c(1,2,3, 4, 5, 6)]),
                           ##rainbow(16, start=.4, end=.1),#
                           xwidth.scale=5,
                           ntrans=1,
                           legend.plotted=TRUE,
                           hist.plotted=FALSE,
                           image.plotted=TRUE,
                           para.plotted=FALSE,
                           lines.plotted=TRUE,
                           lwd.vec=1:7,
                           lty.vec=rep(1,7),
                           col.vec=7:1,
                           range.image=c(0,dim(x)[1]),
                           shrink.legend=TRUE,
                           horizontal.legend = TRUE,
                           offset.legend=0.03,
                           nlevel.legend=length(color.image),
                           xlab.image="",
                           ylab.image="Bins",
                           MY.DEBUG=TRUE,...){
    range.var <-range(x)
    break10 <- defineBreak(x,num.bins,break10=break10,MY.DEBUG=TRUE)
    max.lwd <- max(lwd.vec)
    variable.names <- colnames(x)
    nvar <- dim(x)[2]

    if (nvar < 2){
        stop("nvar should be equal to or greater than 2.")
    }
    
    
    imageBlockinfo <- imageBlock(x=x,num.bins=num.bins,ntrans=ntrans,xwidth.scale=xwidth.scale,hist.plotted=hist.plotted,breaks=break10)
    image.block <- imageBlockinfo[[1]]
    coords <- imageBlockinfo[[2]]
    V1.hist <- imageBlockinfo[[3]]

    varlabpos <- vector()
    for (i in 1:nvar){
        curpos <- ifelse(i==1, (ntrans*xwidth.scale)/2, (ntrans*xwidth.scale)/2 + 2*(i-1)*ntrans*xwidth.scale )
        varlabpos <- c(varlabpos, curpos)
    }

    if (image.plotted==TRUE){
        ## display the image plot
        display.imagepl(image.block,V1.hist,
                        varlabpos,
                        variable.names,
                        zlim=range.image,
                        col=color.image,
                        legend.shrink,
                        horizontal = horizontal.legend,
                        offset=offset.legend,
                        shrink=shrink.legend,
                        nlevel=nlevel.legend,
                        xlab=xlab.image,
                        ylab=ylab.image,
                        main=title)
    } 


    if (joint==FALSE){
        info <- list()
        lines.ls <- list()
        for (varcol in 2:nvar){
            
            first <- varcol-1
            second <- varcol
            
            V1.coord <- coords[,first]
            V2.coord <- coords[,second]
            tab<-table(V1.coord, V2.coord)
            
            
            rows <- sort(unique(V1.coord))
            cols <- sort(unique(V2.coord))
            
            twocoord <- NULL
            for (c in 1:length(cols)){
                for (r in 1: length(rows)){
                    twocoord <- rbind(twocoord, c(rows[r], cols[c]))
                }
            }
            
            colnames(twocoord) <- c(paste("V", first, ".bin", sep=""), paste("V", second, ".bin", sep=""))
            
            pos <- which(tab!=0)
            num.lines <- tab[pos]       
            coords.nozero <- twocoord[pos,]
            
            info[[varcol-1]] <- cbind(coords.nozero, num.lines)
            rownames(info[[varcol-1]]) <- NULL
            lines.ls[[varcol-1]]<- num.lines
        }
    }

    if (joint==TRUE){
        
        ## sort the last element of the list
        sort.pos <- sort(coords[[nvar]], decreasing=FALSE, index.return=TRUE)$ix
        coords <- sortL(coords, sort.pos)
        tab <- table(coords)
        coords.mat <- matrix(unlist(coords), ncol=nvar)
        var.pat <- unique(coords.mat)
        
        dim.name <- labels(tab)
        num.types <- lapply(dim.name, function(w){length(w)})
        ## get the position of the row in the
        ## corresponding element in the list dim.name
                
        dimensions <- t(apply(var.pat, 1, function(row){
            pos <- getPos(row, dim.name)
            
        }))
        
        ## convert the dimensions into the real positions in a vector
        ## please note that R converts lists and matrices into vectors
        ## by column, or else this following procedure will be wrong!
        real.pos <- NULL
        for (row in 1:(dim(dimensions)[1])){
            value <- dimensions[row,1]
            mult <- 1
            for (i in 1:(length(num.types)-1)){
                mult <- mult*num.types[[i]]
                value <- value + (dimensions[row,i+1]-1)*mult
            }
            real.pos <- c(real.pos, value)
        }
        
        ## transform the table into a vector: R does this
        ## by column (not by row...) using as.vector
        
        tab.vec <- as.vector(tab)
        num.lines <- tab.vec[real.pos]
        
        line.info<- cbind(var.pat, num.lines)
        
        
        info <- list()
        lines.ls <- list()
        first.var <- line.info[,1]
        
        for (pair in 2:(dim(var.pat)[2])){
            second.var <- line.info[,pair]
            info[[pair-1]] <- cbind(first.var, second.var, num.lines)
            lines.ls[[pair-1]] <- num.lines
            first.var <- second.var
        }
    }
    ## get the max number of lines
    max.size <- max(num.lines)
    min.size <- min(num.lines)

    
    plotLine(max.size,min.size,info=info,varlabpos,lwd.vec,lty.vec,col.vec,lines.plotted=lines.plotted,lines.ls,image.plotted=image.plotted)


    if (legend.plotted) {
        plot(1:range.var[2],1:range.var[2], xlab="", ylab="", type="n", main="Key for counts signified by the line", xaxt="n", yaxt="n" )
        legend(range.var[1], range.var[2], paste(cts.ls, "<= Counts <=", cts.ls2), lwd=lwd.vec, col=col.vec, lty=lty.vec, bg=color.image[1])
    }    
    ## real parallel coordinates
    if(para.plotted==TRUE){
        parallelCoordinates(x,
                             varlabpos=varlabpos, variable.names=variable.names,
                             at.y=round(V1.hist$breaks, 1)[-1], superimpose=FALSE)
        
        box()
        title(main = title, font.main = 4)
        abline(v=varlabpos)
    }
    
    if (is.null(info)==TRUE){
        return(list(image.block=image.block, breaks=break10))
    } else{
        return(list(image.block=image.block, line.info=info, breaks=break10))
    }
}
 

############################################################################################################################################

defineBreak <- function(x,num.bins,break10=NULL,MY.DEBUG=TRUE){

    if (is.null(break10)){
        break10 <- seq(range.var[1]-1, range.var[2], by=(range.var[2]/num.bins)) 
    }
   ## num.bins is the number of bins for the image plot
   if(num.bins!=(length(break10)-1)){
       if (MY.DEBUG){
           cat("\n\n", "Number of bins, num.bins, has been changed from", num.bins, "to",
               length(break10)-1, "\n\n")
       }
       num.bins <- length(break10)-1
   }
   
   if( break10[length(break10)]<range.var[2] ){
       if (MY.DEBUG){
           cat("\n\n", "The following breaks for the binning:","\n")
           print(break10)
           cat("should be inclusive of the range:", "\n")
           print(range.var)
       }
       seq1 <- seq(range.var[1]-1, range.var[2], by=range.var[2]/num.bins)
       break10 <- c(seq1[-length(seq1)], range.var[2])
       if (MY.DEBUG){
           cat("\n\n", "Thus, the breaks, break10, has been changed to:", "\n")
           print(break10)
       }
   }
    return(break10)
   
}

imageBlock <- function(x,num.bins,ntrans,xwidth.scale,hist.plotted=hist.plotted,breaks){
     image.block <- coords<- NULL
     nvar <- dim(x)[2]
     variable.names <- colnames(x)
     ## check that ntrans is bigger than 1
     if (ntrans <1) {
         stop("ntrans should be greater than or equal to 1")
     }
  
     trans.ct <- rep(0, num.bins*xwidth.scale) ## initialize the transition count to zeros

     V1 <- x[, 1]
     V1.hist <- hist(V1, main=paste("Histogram of", variable.names[1], sep=" "),
                    xlab=variable.names[1], breaks=breaks, plot=hist.plotted, freq=TRUE)
     V1.ct <- V1.hist$counts
     V1.block<-matrix(rep(V1.ct, ntrans*xwidth.scale), ncol=ntrans*xwidth.scale)
     xcoord <- 1:(2+ntrans)
     V1.xpos <- 1
     V1.breaks <- V1.hist$breaks[-1]

     for (second in 2:nvar){
         
       V2 <- x[, second]
       ## histograms with 10 bins
       V2.hist <- hist(V2, main=paste("Histogram of", variable.names[second], sep=" "),
                       xlab=variable.names[second], breaks=breaks, plot=FALSE)
       V2.ct <- V2.hist$counts
    
       ## make the variable image blocks
       V2.block<-matrix(rep(V2.ct, ntrans*xwidth.scale), ncol=ntrans*xwidth.scale)

       ## for each cell/row observation make a slope line formula and
       ## predict the vertical bins (y-coordinate) that the observations
       ## lie depending on the position of the horizontal bin (x-coordinate)
       
       ## make x-coordinates
       ## only does 2 variables right now
       V2.xpos <- length(xcoord)
       xdiff <- V2.xpos-V1.xpos
  
       ## predict the y-coordinates from the x-coordinates
       ycoord.trans <- apply(cbind(V1, V2), 1, function(row){
           ydiff <- row[2]-row[1]
           slope <- ydiff/xdiff
           ypred <- sapply(xcoord[-c(V1.xpos, V2.xpos)], function(x){
               return(slope*(x-V1.xpos)+row[1])
           })
           return(round(ypred,0))
       })
    
       ycoord.trans <- matrix(unlist(ycoord.trans), ncol=length(xcoord[-c(V1.xpos, V2.xpos)]), byrow=TRUE)
       ## get the trans hist for each trans
       trans.block <- NULL
       for (i in 1:ntrans){
           trans.hist <- hist(ycoord.trans[,i], main=paste("Histogram of trans", i, sep=""),
                              xlab=variable.names[2], breaks=breaks, plot=hist.plotted, freq=TRUE)
           trans.block <- cbind(trans.block, matrix(rep(trans.hist$counts, xwidth.scale), ncol=xwidth.scale))
       }
       
       if (hist.plotted==TRUE){
           V2.hist <- hist(V2, main=paste("Histogram of", variable.names[second], sep=" "),
                           xlab=variable.names[second], breaks=break10, plot=TRUE, freq=TRUE)
           
       }
       ## make the total image block
       image.block <- cbind(image.block, V1.block, trans.block)

       V2.breaks <- V2.hist$breaks[-1]
       ## to center the coordinates in the blocks
       V1.coord <- getcoord(V1, V1.breaks) +0.5
       V2.coord <- getcoord(V2, V2.breaks) + 0.5
       coords <- cbind(coords, V1.coord)
       V1 <- V2
       
   }
     image.block <- cbind(image.block, V2.block)
     coords <- cbind(coords, V2.coord)
     
     return(list(image.block, coords,V1.hist))
 }


getcoord <- function(V, V.breaks){
    V.coord <- NULL
    for (i in 1:(length(V))) {
        ## j is the bin number for each element of V
        j <- 1
        while ((j < length(V.breaks)) & (V[i] >= V.breaks[j])){
            j <- j+1
        }
        
        V.coord <-c(V.coord, j)
        
    }
    return(V.coord)
}


 
display.imagepl <- function(image.block,V1.hist,
                            varlabpos,
                            variable.names,
                            zlim=range.image,
                            col=color.image,
                            legend.shrink,
                            horizontal = horizontal.legend,
                            offset=offset.legend,
                            nlevel=nlevel.legend,
                            shrink=shrink.legend,
                            xlab,
                            ylab,
                            main,...){
     ## coordinates of the image block
    col.coord<-1:(dim(image.block)[2]+1)
    row.coord <- 1:(dim(image.block)[1]+1)

    image(col.coord, row.coord, t(image.block), zlim=zlim, col=col, axes=FALSE,
          xlab=xlab, ylab=ylab, ...)
    lib.there <- require(fields)
    if (lib.there==FALSE){
        warning("The function image.plot in Library fields is needed for the image parallel coordinates plot to have a color legend.")
    } else {
        image.plot(col.coord, row.coord, t(image.block), zlim =zlim, legend.only = TRUE, legend.shrink=shrink,
                   horizontal = horizontal, col=col, offset=offset, nlevel=nlevel, axes=FALSE,
                   xlab=xlab, ylab=ylab, ...)
    }
    
    axis(2, at=1:length(V1.hist$breaks), lab=round(V1.hist$breaks, 1))
    axis(1, at=varlabpos, lab=variable.names)
    box()
    title(main, font.main = 4)
    abline(v=varlabpos)
}


plotLine <- function(max.size,min.size,varlabpos,info,lwd.vec,lty.vec,col.vec,lines.plotted=lines.plotted,lines.ls,image.plotted=lines.plotted){

    scaled.size.ls <- NULL
    max.lwd <- max(lwd.vec)
    
    ## plotting of the lines from the last variable to the first
    for(i in length(info):1){
        ## ordering to plot the large num lines first
        kk <- info[[i]]
        rownames(kk) <- NULL
        order.kk <- sort(kk[,3], decreasing=FALSE, index.return=TRUE)$ix
        kk <- kk[order.kk,]
        
        ## actual plotting of the lines
        scaled.size.el <- apply(kk, 1, function(y){
            scaled.size <- (max.lwd-1)*((y[3]-min.size)/(max.size-min.size))+1
            lwd.current <- round(scaled.size,0)
            pos <- which(lwd.vec==lwd.current)
            
            if (lines.plotted==TRUE) {
                if (image.plotted ==TRUE) {
                    lines(varlabpos[c(i,i+1)], c(y[1], y[2]), lwd=lwd.current,
                          col=col.vec[pos], lty=lty.vec[pos])
                } else {
                    warning("'lines.plotted'=TRUE, but lines could not be plotted because 'image.plotted'=FALSE")
                }
            }
            return(round(scaled.size, 0))
        })
        ##go back to the original order
        order.orig <- sort(order.kk, decreasing=FALSE, index.return=TRUE)$ix
        scaled.size.ls <- c(scaled.size.el[order.orig], scaled.size.ls)
    }
    lines.ls <- unlist(lines.ls)
    names(lines.ls) <- NULL
    
    cts.ls <- cts.ls2 <- NULL
    for (j in 1:max.lwd){
        if (is.element(j, scaled.size.ls)==TRUE){
            cts.max <-max(lines.ls[scaled.size.ls==j])
            cts.min <- min(lines.ls[scaled.size.ls==j])
            cts.ls <- c(cts.ls, cts.min)
            cts.ls2 <- c(cts.ls2, cts.max)
        } else {
            cts.ls <- c(cts.ls, "--")
            cts.ls2 <- c(cts.ls2, "--")
        }
    }
    
}

    
########## Joint 
sortL <- function(coords, sort.pos){
    lapply(coords, function(w){w[sort.pos]})
}

getPos <- function(row, dim.name){
    info <- cbind(row, 1:length(row))
    pos <- apply(info, 1, function(w){
        which(w[1]==as.numeric(dim.name[[w[2]]]))
    })
    return(unlist(pos))
    
}
